/*************** PRINT/DISPLAY/RS-232 ROUTINES *******************/
#include	"display.h"
#include 	"stdlib.h"
#include 	"string.h"
#include 	"delays.h"
#include 	"usart.h"
#include 	"images.h"
//--------------------------------------------------------------------------	
void fp(int cfield, int cplaces)
{
	gcField = (unsigned char)cfield;		//Total field width including '.'
	gcPlaces = (unsigned char)cplaces;		//No. of digits after decimal point
}
//--------------------------------------------------------------------------
void ftoa (char *buf, double fval, int cField, int cPlaces)
{
	//This routine correctly formats a floating point number into it's ASCII representation.
	//The routine will insert leading spaces as necessary to satisfy the requirements of the
	//specified formatting.
	//
	//eg:
	//ftoa(buf,12.34567,5,3);		//Ask for 5 digit field including '.',  and 3 digits after.
	//After the routine, buf will contain: '12.34'.
	//
	//This allows for easier integration of displays onto the VFD.
	//

	long lTemp;
	long scale;
	int i;
	char dbuf[8];	//Temporary - used and overwritten in routine
	char pbuf[8];	//Temporary - used and overwritten in routine
	char fbuf[16];	//The main 'build' buffer.  The output string is built here
					//and copied into the *buf buffer at the end.
	char cDigits;
	size_t szTemp;

	if(cPlaces)
	{
		cDigits=cField-cPlaces-1;	//FP formatting.  Includes '.' and decimal part
	}
	else
	{
		cDigits=cField;			//Special case for integers
	}
	
	//EXAMPLE USED: fval=123.456789, fp(5,5) has been set
	memset(fbuf,0,sizeof(fbuf));	//Clear the buffer
	scale=1;					//Set default scale value

	lTemp=(long)fval;			//convert double to long	- ltemp=123
	ltoa(lTemp,dbuf);			//convert long to ascii in dbuf
	szTemp=strlen(dbuf);		//How many digits does the number use?


	if((char)szTemp<cDigits)	//Do we need leading whitespaces?
	{
		for(i=0; i<cDigits-(int)szTemp+0; i++)
		{
			pbuf[i]=' ';		//Insert the reqd number of spaces into the buffer 'pbuf'
		}
		
		if((fval<0) && (fval>-1) )  //Special case for -0.1 (etc) where 'digit' is = 0, but fval is -ve.
		{
			pbuf[i-1]='-';		//Add -ve symbol to the main buffer
		}
		pbuf[i]=0;				//Null terminate the spaces so strcat can find it
								//No need to add '1' - the for loop does this for us!
		strcat(fbuf,pbuf);		//Add the spaces (and sign?) into the main buffer
	}
	else
	{
		if((fval<0) && (fval>-1) )  //Special case for -0.1 (etc) where 'digit' is = 0, but fval is -ve.
		{
			fbuf[0]='-';				//Add -ve symbol to the main buffer
			fbuf[1]=0;				//Null terminate
		}
	}
	strcat(fbuf,dbuf);			//Add the numerical part to the main buffer (after '-' and spaces if reqd)
	
	if(gcPlaces)				//Do we need the decimal part?
	{
		strcatpgm2ram(fbuf,(rom char *)".");		//add '.' to end of main buffer
					
		
		//Now scale up the decimal part...
		
		//for(i=1; i<=giPlaces; i++)
		//{
		//	scale=scale*10;			//work out the scale according to 'giPlaces'
		//}							//eg with fp(5,5) scale=100000
		//fval=fval*scale;			//scale up decimal part - fval=45678.9
		switch(cPlaces)
		{
			case 1:	scale=10;	  break;
			case 2:	scale=100; 	  break;
			case 3: scale=1000;	  break;
			case 4:	scale=10000;  break;
			case 5: scale=100000; break;
		}

		fval=fval-(int)fval; 		//Remove integer part... fval=0.456789
		fval*=scale;
		lTemp=(long)fval;			//convert to long integer - ltemp=45678
		
		if(lTemp<0) lTemp=-lTemp;	//No negative fractions!!! (prevents -1.-234 etc)
		ltoa(lTemp,dbuf);			//copy into dbuf - dbuf='45678'
		strcat(fbuf,dbuf);			//add the numerical part to the main buffer
	}
	
	strncpy(buf,fbuf,(size_t)cField);		//copy the final result to the output buffer
	buf[cField]=0;					//Null terminate buf
									
}	
//--------------------------------------------------------------------------
void wait_ms(int i)
{  
	//Delay10KTCYx() waits for multiples of 10000 instructions.
	//At 8MHz, 1 TCY is 500nS, so each 2 x 1KTCY is 1mS.
	//PORTCbits.RC2=0;		//System LED on
	while(i--)
	{
		Delay1KTCYx(2); 	//delay 'i' mS
	}
	//PORTCbits.RC2=1;		//System LED off
	//Delays are not exact due to overheads...
}

//--------------------------------------------------------------------------
void wait_us(int i)
{  
	//Delay10TCYx() waits for multiples of 10 instructions.
	//At 20MHz, 1 TCY is 200nS, so each 5TCY is 1uS.
	//PORTCbits.RC2=0;		//System LED on
	while(i--)
	{
		Nop();
		Nop();
//		Nop();
//		Nop();
//		Nop();
	}
	//PORTCbits.RC2=1;		//System LED off
	//Delays are not exact due to overheads...
}

//--------------------------------------------------------------------------
char BusyCheck(void)  
{
	if(PORTCbits.RC1)
	{
		return 1;
	}else{
		return 0;
	}
}
//--------------------------------------------------------------------------
void ControllerCMD(char val)  
{
//	12	RC1		MB HIGH = VFD BUSY (PIC INPUT)
//	13	RC2		HB HIGH = PIC BUSY (PIC OUTPUT)
	while(PORTCbits.RC1);
	while(BusyUSART());
	putcUSART(0x0F);	//Control character
	while(PORTCbits.RC1);
	while(BusyUSART());
	putcUSART(val);		//Write command
	while(PORTCbits.RC1);
}
//--------------------------------------------------------------------------
void ControllerDATA(char val)  /* new controller */
{
//	12	RC1		MB HIGH = VFD BUSY (PIC INPUT)
//	13	RC2		HB HIGH = PIC BUSY (PIC OUTPUT)
	while(PORTCbits.RC1);
	while(BusyUSART());
	putcUSART(val);		//Write data
	while(PORTCbits.RC1);
}
//--------------------------------------------------------------------------
void init_lcd()
{
	// Display self initialises in serial mode...
	// Setup RS232 comms 19200,N,8,1
	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_OFF & USART_ASYNCH_MODE & USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH, 25);
}

//--------------------------------------------------------------------------
void ShowMeter(char cLevel)
{
	//cLevel is from 0 to 40

	unsigned char n;
	char rem;
	if(cLevel>138) 
	{
		cLevel=138;
	}else{
		ClearBox(cLevel+1,1,138,4,1);
	}
	DrawBox(0,0,cLevel,5,1);
	/*
	if(cLevel>130)
	{
		strcpypgm2ram(buf,(rom char *)"<CLIP>");
		printat(34,buf);
		//putcharat(34,4);
		//putcharat(35,5);
		//putcharat(36,6);
		//putcharat(37,7);
		
	}else{
		strcpypgm2ram(buf,(rom char *)"      ");
		printat(34,buf);
	}
	*/
	/*
	if(cLevel>40) cLevel=40;
	rem=0x20;//0x03;					// CG char - empty space
	n=0;	
	if(cLevel & 1) rem='I';//0x02;	//CG char - single vertical
	cLevel/=2;
	while(n<cLevel)
	{
		buf[n++]='W';//0x01;			//CG char - double vertical
	}
	buf[n]=rem;					//add on remainder (either single bar or space)
	n++;
	while(n<20)
	{
		buf[n++]=0x20;//0x03;			//Fill up the rest with empty spaces
	}
//	buf[19]=0x04;				//End with a bar?
	buf[20]=0;					//and null terminate
	print_top(buf);
	*/
}
//--------------------------------------------------------------------------
void DrawBox(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, int iFill)
{
	ControllerCMD(0xf1);
	ControllerDATA(x1);
	ControllerDATA(y1);
	ControllerDATA(x2);
	ControllerDATA(y2);
	if(iFill)
	{
		ControllerDATA('F');
	}
	else
	{
		ControllerDATA('O'); //outline only
	}		
}
//--------------------------------------------------------------------------
void ClearBox(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, int iFill)
{
	ControllerCMD(0xf1);
	ControllerDATA(x1);
	ControllerDATA(y1);
	ControllerDATA(x2);
	ControllerDATA(y2);
	if(iFill)
	{
		ControllerDATA('C');
	}
	else
	{
		ControllerDATA('o'); //outline only
	}		
}
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
void DisplayLogo(unsigned char image)
{
	int i;
	unsigned char temp;
	
	clear_screen();
	ControllerCMD(0xf1);
	ControllerDATA(0);
	ControllerDATA(0);
	ControllerDATA(139);
	ControllerDATA(15);
	ControllerDATA('v');
	switch(image)
	{
		case 0:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=edsbassrig[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 1:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=mosquito[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 2:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=madinventions[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 3:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=meatisdinner[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 4:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=lights[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 5:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=ewoowoo[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 6:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=fender[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 7:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=cocktail[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 8:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=ginsters[i];
				ControllerDATA(temp);
			}
			break;
		}
		case 9:
		{
			for(i=0; i<280; i++)
			{
				//memcpypgm2ram(temp,logo[i],1);
				temp=pies[i];
				ControllerDATA(temp);
			}
			break;
		}
	}
}
//--------------------------------------------------------------------------
void SetFont(char f)
{
	ControllerCMD(0xf2);
	ControllerDATA(f);
}
//--------------------------------------------------------------------------

void conv(int *pos)
{  if (*pos<0x80)
   {  if (*pos<=20) *pos += 0x7f;
       else *pos += 171;
   }
}
//--------------------------------------------------------------------------
void cursor(int pos)
{  if (pos<128)
   {  if (pos<=20) pos=pos+0x7f;
       else pos=pos+171;
   }
   ControllerCMD(pos);
   wait_us(200);
}
//--------------------------------------------------------------------------
void putcharat(int pos,int c)    /* 4 bit */
{ 
	ControllerCMD(0xf0);
	if(pos<128)
	{	if(pos<=20) 
		{
			ControllerDATA((pos-1)*6);
			ControllerDATA(0x07);
		}
		else 
		{
			ControllerDATA((pos-21)*6);
			ControllerDATA(0x0f);
		}
	}
   	
	if (c=='{') c=LARROW; if (c=='}') c=RARROW;
   	ControllerDATA(c); //wait_ms(10);
}
//--------------------------------------------------------------------------
void printat(int pos,char *s)   /* 4 bit data */
{  
	char c; int count;
   	count=0;
	ControllerCMD(0xf0);
	if(pos<128)
	{	if(pos<=20) 
		{
			ControllerDATA((pos-1)*6);
			ControllerDATA(0x07);
		}
		else 
		{
			ControllerDATA((pos-21)*6);
			ControllerDATA(0x0f);
		}
	}

   //wait_ms(1);
   //Delay 6000 instructions = 1mS.
   Delay1KTCYx(15);
   for (;(*s);s++)
   {  c=*s;
      if (c=='{') c=LARROW; if (c=='}') c=RARROW;
      ControllerDATA(c);
      wait_us(70);
      //wait_ms(10);
      count++; if (count==20) break;
   }
}
	
//--------------------------------------------------------------------------
void SetCursorType(int code)
{  switch (code)
   {  case 0: ControllerCMD(12); break; /* off   */
      case 1: ControllerCMD(14); break; /* line  */
      case 2: ControllerCMD(13); break; /* blink */
   }
}
//--------------------------------------------------------------------------
void print_top(char *s)
{  printat(1,s);
}
//--------------------------------------------------------------------------
void print_bot(char *s)
{  printat(21,s);
}
//--------------------------------------------------------------------------
void print_top_rom(rom char*s)
{
	strcpypgm2ram(buf,s);
	print_top(buf);
}
//--------------------------------------------------------------------------
void print_bot_rom(rom char*s)
{
	strcpypgm2ram(buf,s);
	print_bot(buf);
}
//--------------------------------------------------------------------------
//ram char clearline[] = {"                "};
//--------------------------------------------------------------------------
void clear_top(void)
{ 	
//	strcpypgm2ram(buf,(rom char *)"                    ");
//	printat(1,buf);
	ClearBox(0,0,112,7,1);
//	ControllerCMD(0xf1);
//	ControllerDATA(0);
//	ControllerDATA(0);
//	ControllerDATA(112);
//	ControllerDATA(7);
//	ControllerDATA('C');

}
//--------------------------------------------------------------------------
void clear_bot(void)
{
//	strcpypgm2ram(buf,(rom char *)"                    ");
//	printat(21,buf);
	ClearBox(0,8,112,15,1);
//	ControllerCMD(0xf1);
//	ControllerDATA(0);
//	ControllerDATA(8);
//	ControllerDATA(112);
//	ControllerDATA(15);
//	ControllerDATA('C');
}
//--------------------------------------------------------------------------
void clear_screen(void)
{  
	ControllerCMD(1);
	clear_top(); 
   	clear_bot(); // wait_ms(1000)
}

//--------------------------------------------------------------------------
void fprintat(int pos,double fv, int cfield, int cplaces)
{  
	ftoa(buf,fv,cfield,cplaces);
	//convert(fv,buf);
    printat(pos,buf);
}
//--------------------------------------------------------------------------
void n2printat(int pos,int ival)
{  
	char tbuf[8];
	double tfval;
	
	tfval=(int)ival;
    ftoa(tbuf,tfval,2,0);
    if (tbuf[0]==SP) 
	{ 
		tbuf[0]=tbuf[1]; 
		tbuf[1]=SP; 
	} 
    printat(pos,tbuf);
    cursor(pos);
}
//--------------------------------------------------------------------------
void nprintat(int pos,int ival, int cfield)
{  
	char tbuf[8];
	double tfval;
	tfval=(int)ival;
    ftoa(tbuf, tfval, cfield, 0);
    printat(pos,tbuf);
}
//--------------------------------------------------------------------------

void set_display_brightness(int code)
{  
	ControllerCMD(0x30);
   	ControllerDATA(code);
}
//--------------------------------------------------------------------------
int streq(auto char *p,auto char *q)
{  while(*p)
   {  if (*p != *q) return(0);
      p++; q++;
   }
   if (*q == 0) return(1);
   return(0);
}
//--------------------------------------------------------------------------
void strcpyn(char *p,char *q)
{  while (*q)
   { *p=*q; p++; q++;
   }
   *p=0;
}
//--------------------------------------------------------------------------
void trx(int c)
{  	
	//Wait until trx is ready
	while(BusyUSART());
	WriteUSART((char)c);
}
//--------------------------------------------------------------------------
void copy(char *b,const char *s)
{  
	while(*s) { *b=*s; b++; s++; }
}
//--------------------------------------------------------------------------
void ncopy(char *b,int n)
{  int j;
   if (n<100)
   {  j=n/10;
      if (j==0) b[0]=' '; else b[0]=j+'0';
      b[1]=n-j*10+'0';
   }
   else
   {  j=n/100;
      b[0]=j+'0';
      n=n-j*100;
      j=n/10;
      b[1]=j+'0';
      b[2]=n-j*10+'0';
   }
}
//--------------------------------------------------------------------------
//RS-232 SEND POSITION

void sendx(char *s)    /* send without leading spaces */
{  while (*s)
   {  if (*s != SP) WriteUSART(*s);
      s++;
   }
}
//--------------------------------------------------------------------------
void send(char *s)
{  
	putsUSART(s);
	//while (*s)
   	//{  Write1USART(*s);
   	//   s++;
    //	}
}
//--------------------------------------------------------------------------
/*
void send_coord(void)
{  send(buf); trx(CR); 
}
//--------------------------------------------------------------------------
void send_pos(void)
{  calc_pos();
   //fp(7,3);
   convert(cx,buf); 
   send_coord();
   convert(cy,buf); 
   send_coord();
   trx(CR);  
   key=0;
}
//--------------------------------------------------------------------------
void send_dir(int code)  // 1=X 2=Y 4=Z 8=all 
{  int c;
   c='Y';
   if ((code & 1) && dirx) c='N';
   if ((code & 2) && diry) c='N';
   if (rz && (code & 4) && dirz) c='N';
   if (code==8)
   {  if (read_from_bus(limit) & 15) c='L';
       else c='N';
   }
   if ((code & 1) && (read_from_bus(limit) & 3)) c='L';
   if ((code & 2) && (read_from_bus(limit) & 12)) c='L';
   trx(c);
}
*/
/*************************** END *****************************/
